// CustomDraw.cpp: implementation of the CCustomDraw class.
//
//////////////////////////////////////////////////////////////////////
#include "StdAfx.h"
#include "resource.h"
#include "CustomDraw.h"
#include "Types.h"
#include "AfxCmn.h"

#include <Dapfor/Common/DataObject.h>

#include <Dapfor/GUI/Column.h>
#include <Dapfor/GUI/GridCell.h>
#include <Dapfor/GUI/GridLine.h>
#include <Dapfor/GUI/MsImg32.h>
#include <Dapfor/GUI/PaintContext.h>
#include <Dapfor/GUI/Grid.h>

CExplorerCustomDraw::CExplorerCustomDraw(bool bAutoDelete) 
 : Dapfor::GUI::CCustomDraw(bAutoDelete) 
{
    Dapfor::GUI::CMsImg32::Load32BitImage(m_BuilletTriangle, IDB_BULLET_TRIANGLE, true, RGB(255, 0, 255));
}

void CExplorerCustomDraw::DrawCell(Dapfor::GUI::CGridCell& cell, const Dapfor::GUI::CPaintContext& paintContext, UINT paintFilter)
{
    //This code customizes the drawing. Note that we do not use the casting. 
    //We have an access to any information of our business object.
    //Also you can see, that the data (folders, files...) is clearly separated from the presentation (icons, text, etc...)

    //Get the business object (drive, folder or file)
    const Dapfor::Common::CDataObject* pDO = cell.GetGridLine().GetDataObject();
   
    //Set individual colors for the columns
    if(cell.GetColumn())
    {
        switch(cell.GetColumn()->GetFid())
        {
            case FidSize:
                cell.GetPaintInfo().SetBackColor(RGB(250, 246, 232));
                break;

            case FidFilesCount:
                cell.GetPaintInfo().SetBackColor(RGB(247, 243, 221));
                break;

            case FidCreationTime:
                cell.GetPaintInfo().SetBackColor(RGB(245, 239, 209));
                break;

            case FidLastAccessTime:
                cell.GetPaintInfo().SetBackColor(RGB(240, 232, 193));
                break;

            case FidLastWriteTime:
                cell.GetPaintInfo().SetBackColor(RGB(235, 225, 177));
                break;
        }
    }



    //Check whether the business object and column are not null. 
    //Check if the field identifier is correspond to the "Name" column
    if(pDO != 0 && cell.GetColumn() && cell.GetColumn()->GetFid() == FidName)
    {

        //Get the full name of the drive, folder or file
        CString name = pDO->GetMfcString(FidFullName);
        
        //Get an image list and index for the specified business object
        SHFILEINFO  sfi;
        HIMAGELIST himl = (HIMAGELIST)::SHGetFileInfo(name, 0, &sfi, sizeof(sfi), SHGFI_SYSICONINDEX);
        
        if(himl != 0)
        {
            m_ImageList.Attach(himl);

            //Set image list and index for the given cell
            cell.SetImageId(sfi.iIcon);
            cell.SetImageList(&m_ImageList);
            cell.SetStretchImage(true);
        }

        //We will paint files with the blue color and folders with the black one
        if(pDO->GetLong(FidItemType) == FileData)
        {
            cell.GetPaintInfo().SetForeColor(RGB(62, 28, 196));
        }


    }

    //This code demonstrated how to recursively change an icon for the cell
    if(pDO != 0 && cell.GetColumn() && cell.GetColumn()->GetFid() == FidSize)
    {
        if(pDO->GetBool(FidIsPopulating))
        {
            //Select an icon from the image list
            cell.SetImageId((GetTickCount() % 4000) / 1000);
            cell.SetImageList(&m_BuilletTriangle);
            cell.SetStretchImage(false);
            
            // This line asks for the next painting routine for the given cell in 30 msec.
            cell.RequestNextPaint(30); 
        }
    }

    
    //Change font for the files and folders...
    CFont font;
    if(CFont* cellFont = cell.GetPaintInfo().GetFont())
    {
        if(pDO != 0 && cell.GetColumn())
        {
            LOGFONT lf;
            cellFont->GetLogFont(&lf);
            
            if((pDO->GetLong(FidItemType) == FileData) && 
               (cell.GetColumn()->GetFid() == FidName))
            {
                lf.lfItalic = true;
            }
            if(cell.GetColumn()->GetFid() == FidSize)
            {
                lf.lfWeight = 1000;
            }

            font.CreateFontIndirect(&lf);
            cell.GetPaintInfo().SetFont(&font);
        }
    }


    //We have prepared parameters for drawing. Now, we can use the default implementation to draw a cell
    CCustomDraw::DrawCell(cell, paintContext, paintFilter);
    
    //Free already taken resources.
    m_ImageList.Detach();
}

// Computes an optimal width of a cell taking into account a font, hierarchy, text size...
int CExplorerCustomDraw::GetOptimalCellWidth(Dapfor::GUI::CGridCell& cell, const Dapfor::GUI::CPaintContext& paintContext, UINT paintFilter) const
{
    const Dapfor::Common::CDataObject* pDO = cell.GetGridLine().GetDataObject();
    const Dapfor::GUI::CColumn* column = cell.GetColumn();
    if(pDO && column)
    {
        switch(column->GetFid())
        {
        case FidName:
            return BaseClass::GetOptimalCellWidth(cell, paintContext, paintFilter) + 
                    cell.GetPaintInfo().GetVirtualRect().Height();
    

        case FidSize:
            //Adjust column's size for the bold font
            {
                int size = 0;
                CFont font;
                if(CFont* cellFont = cell.GetPaintInfo().GetFont())
                {
                    LOGFONT lf;
                    cellFont->GetLogFont(&lf);
                    lf.lfWeight = 1000;
                    font.CreateFontIndirect(&lf);
                    cell.GetPaintInfo().SetFont(&font);

                }
                //Compute size with the new font
                size = BaseClass::GetOptimalCellWidth(cell, paintContext, paintFilter);

                //Don't forget to add a size for the icon
                if(pDO->GetBool(FidIsPopulating))
                {
                    size += cell.GetPaintInfo().GetVirtualRect().Height();
                }
                return size;

            }
        }
    }
    return BaseClass::GetOptimalCellWidth(cell, paintContext, paintFilter);
}



