// FileManager.cpp: implementation of the CFileManager class.
//
//////////////////////////////////////////////////////////////////////
#include "StdAfx.h"
#include "FileManager.h"
#include "FolderData.h"
#include "FileData.h"
#include <iostream>
#include <algorithm>

#include <Dapfor/Common/Lock.h>
#include <Dapfor/Common/Int64DateFormat.h>


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CFileManager::CFileManager()
{

}

CFileManager::~CFileManager()
{

    for(Disks::iterator it = m_Disks.begin(); it != m_Disks.end(); ++it)
    {
        delete *it;
    }
}

int CFileManager::Run()
{

    BuildDrives();

    return 0;
}

void CFileManager::BuildDrives()
{
    //To avoid sending notifications inside of the lock, we will copy all listeners.
    //This is not efficient, but thread safe.
    Listeners listenersCopy;
    {
        //Take the lock locally that protect the copy operation.
        Dapfor::Common::CLock lock(m_Synchro);
        
        //Copy all listeners
        listenersCopy = m_Listeners;
    }



    //Get a bitset of all drives
    DWORD dwLogicalDrives = ::GetLogicalDrives();
    if(dwLogicalDrives)
    {
        for(int iCnt = 0; iCnt < 32; ++iCnt)
        {
            // Set driveName root
            CString driveName;
            driveName.Format(_T("%c:"), iCnt + _T('A'));

            // Determine partition type
            UINT uiDriveType = ::GetDriveType(driveName);
            switch(uiDriveType)
            {
                // Unknown
            case DRIVE_UNKNOWN:
            case DRIVE_NO_ROOT_DIR:
            case DRIVE_REMOVABLE:
            case DRIVE_REMOTE:
            case DRIVE_CDROM:
                break;
                
                // Fixed or RAM drives
            case DRIVE_FIXED:
            case DRIVE_RAMDISK:
                {
                    CFolderData* drive = new CFolderData(driveName, 0);
                    
                    //We suppose, that insertion happes in the single thread. Otherwise it should to take a lock.
                    m_Disks.push_back(drive);

                    //Send notification to all listeners
                    for(Listeners::iterator it = listenersCopy.begin(); it != listenersCopy.end(); ++it)
                    {
                        (*it)->OnAddDrive(*drive);
                    }
                }
                break;
            }
        }
    }

    //Build folders on each drive
    for(Disks::iterator it = m_Disks.begin(); it != m_Disks.end(); ++it)
    {
        BindFolders(*it);
    }
}

void CFileManager::BindFolders(CFolderData* root)
{
    //Stop processing
    if(IsStopping())
    {
        return;
    }

    //To avoid sending notifications inside of the lock, we will copy all listeners.
    //This is not efficient, but thread safe.
    Listeners listenersCopy;
    {
        //Take the lock locally that protect the copy operation.
        Dapfor::Common::CLock lock(m_Synchro);
        
        //Copy all listeners
        listenersCopy = m_Listeners;
    }

    
    
    if(root != 0)
    {
        CString fullName = root->GetFullName();
        CFileFind ff;
        if(ff.FindFile(fullName + CString("\\*")))
        {
            //Look for each file or subdirectory
            bool lastFile = false;
            while(!lastFile)
            {
                //Stop processing
                if(IsStopping())
                {
                    return;
                }

                lastFile = ff.FindNextFile() == 0;
                if(!ff.IsDots())
                {
                    if(ff.IsDirectory())
                    {
                        CFolderData* child = new CFolderData(ff.GetFileName(), root);
                        
                        //To prevent from sending notifications at each file adding on each level in the hierarchy, 
                        //which is very expensive operation, we'll add subfolder to the root item after it has been built.
                        root->AddSubFolder(child);

                        //Add subfolter to the grid
                        for(Listeners::iterator it = listenersCopy.begin(); it != listenersCopy.end(); ++it)
                        {
                            (*it)->OnAddFolder(*child);
                        }
                    }
                    else
                    {
                        FILETIME creationTime;
                        ff.GetCreationTime(&creationTime);

                        FILETIME lastAccessTime;
                        ff.GetLastAccessTime(&lastAccessTime);

                        FILETIME lastWriteTime;
                        ff.GetLastWriteTime(&lastWriteTime);


                                            
                        CFileData* file = new CFileData(ff.GetFileName(), 
                                                        root, 
                                                        ff.GetLength(),
                                                        Dapfor::Common::CInt64DateFormat::FileTimeToInt64(creationTime), 
                                                        Dapfor::Common::CInt64DateFormat::FileTimeToInt64(lastAccessTime), 
                                                        Dapfor::Common::CInt64DateFormat::FileTimeToInt64(lastWriteTime));
                        
                        root->AddFile(file);
                        
                        //Send notification to all listeners
                        for(Listeners::iterator it = listenersCopy.begin(); it != listenersCopy.end(); ++it)
                        {
                            //Stop processing
                            if(IsStopping())
                            {
                                return;
                            }
                            
                            (*it)->OnAddFile(*file);
                        }
                    }
                }
            }
        }

        root->BeginPopulate();
        //Attach files in subfolders recoursively 
        for(CFolderData::Folders::iterator it = root->GetFolders().begin(); it != root->GetFolders().end(); ++it)
        {
            BindFolders(*it);
        }
        root->EndPopulate();

    }

}



void CFileManager::AddListener(IListener* listener)
{
    if(listener != 0)
    {
        //This protects m_Listeners by the critical section
        Dapfor::Common::CLock lock(m_Synchro);
        
        //Add a new listener
        m_Listeners.push_back(listener);
    }
}

void CFileManager::RemoveListener(IListener* listener)
{
    if(listener != 0)
    {
        //This protects m_Listeners by the critical section
        Dapfor::Common::CLock lock(m_Synchro);
        
        //remove listener
        Listeners::iterator it = std::find(m_Listeners.begin(), m_Listeners.end(), listener);
        if(it != m_Listeners.end())
        {
            m_Listeners.erase(it);
        }
    }
}
