/////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright 2007-2008 Dapfor
//                                                                         
// This file is part of the Dapfor software, and may only be used, modified, and distributed 
// under the terms of the Dapfor Software License, Version 1.0.
// (See accompanying file lisense.txt or open the license at www.dapfor.com/license_agreement.aspx)
// By continuing to use, modify, or distribute this file you indicate that you have read the license 
// and understand and accept it fully.
/////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include <Dapfor/GUI/Types.h>
#include <Dapfor/GUI/GUI_Export.h>
#include <Dapfor/GUI/TooltipSettings.h>
#include <Dapfor/GUI/IEditInPlace.h>
#include <Dapfor/GUI/EditInPlaceSettings.h>
#include <Dapfor/GUI/HighlightSettings.h>
#include <Dapfor/GUI/EmbeddedControlParams.h>
#include <Dapfor/GUI/Appearance.h>
#include <Dapfor/GUI/BackgroundImage.h>
#include <Dapfor/GUI/DragDropSettings.h>
#include <Dapfor/GUI/CopyPasteSettings.h>
#include <Dapfor/GUI/PrintSettings.h>
#include <Dapfor/GUI/Header.h>
#include <Dapfor/GUI/SortInfo.h>

#include <vector>

//Forward declarations
class CImageList;

namespace Dapfor
{

namespace Common
{
class CDataObject;
class CValue;
}

namespace GUI
{
class CHeader;
class IGridFilter;
class ICustomDraw;
class ICopyPaste;
class IDragDrop;
class CDragSource;
class IKeyManager;
class CHighlightSettings;
class CColorBlend;
class ITooltip;
class ITooltipCtrl;
class IForEach;
class CColumn;
class CEmbeddedControlAdapter;

//////////////////////////////////////////////////////////////////////////////////////////////
/// \brief This class implements full featured thread safe hierarchical list control.
///
/// \par Introduction
///
/// The grid is oriented to work with data of practically any types. These data are the objects of 
/// usual C++ classes with the set of Get- and Set- methods that can be called by their identifiers. 
/// These identifiers are declared with a class together. In order to be able to insert such objects 
/// into the grid, the class should inherit from Common::CDataObject which permits 
/// to call the function by its identifier. These objects are displayed in the grid by using the binding mechanism. 
/// The rows of the grid correspond to objects of your classes and columns to their functions.
///
/// The grid provides possibilities of automatic updating, sorting, highlighting and filtration 
/// of the objects at each values modification. To benefit this, it is recommended to implement 
/// Common::CDataOnject::ISink interface that can fire notifications to the grid.
///
/// The grid allows you to use simultaneously objects of different classes. The only condition is that types 
/// of the values returned by the functions should be the same. It is not so critical for 
/// data painting and highlighting but at sorting it is required to compare values of one type.
///
/// \par Purpose of contexts
///
/// The grid has two graphical contexts: fixed and scrollable. The reason why these contexts have been organized 
/// is that it is not possible to set just a number of 'frozen' rows in the hierarchical grid. 
/// Indeed, the quantity of them is not constant and may vary depending on whether the children are expanded 
/// or collapsed. Furthermore data objects can be automatically filtered in the IGridFilter interface.
///
///
/// \par Handles & lines in the grid
///
/// As mentioned above the grid works with C++ objects. One data object can be many times inserted into 
/// one grid. Handles were organized in order to uniquely determine object's location in the grid. 
/// Handles take part in all grid operations. Besides they keep information about hierarchy, selection, 
/// filtration, time of the last updating and so on. A handle may not be always visible on the screen. 
/// It can be filtered by IGridFilter interface or hidden by its parent. So, for such handle the 
/// corresponding grid line does not exist. Handles can be easily detached from the grid with all hierarchy 
/// and inserted into any other place or grid.
///
///
/// \par Focus
///
/// Focus is designed to define the cursor location at navigation and lines selection in the grid. 
/// Only one focused line is permitted in the grid. As the grid can support several contexts then 
/// the focused context is the context where the focused line is located.
///
///
/// \par Appearance
///
/// Grid appearance is stored in an object of CAppearance type. At data drawing all colors are got from this object. 
/// The all painting information is passed through ICustomDraw interface where it can be freely re-defined. 
///
/// \par Tooltips
///
/// A tooltip is designed to enlarge an application functionality for instances when information that must be 
/// provided to the user cannot be displayed entirely. 
/// Functionally the tooltip is divided into two interfaces. The first interface is designed to 
/// manage a pop-up window, its color, location and so on. The second interface is designed to manage 
/// the content displayed in the window. GUI library provides two tooltips: 
/// - A tooltip that shows information in the small popup window as a plain text or an html text in the 
///   popup browser.
/// - Embedded tooltip. In case when some part of a text in a cell becomes invisible then the remaining text 
/// is automatically shown in the tooltip above the cell with the same font.
///
/// \par Copy & paste
///
/// Copy & paste allows you to move data inside one grid, between various grids or between different applications. 
/// The grid allows you to export data to such applications as Excel and Word. In addition to standard clipboard 
/// formats, GUI library has new ones that allow you to maintain a hierarchical information and also to move data 
/// between different applications with data serialization where it is possible. At any moment the programmer 
/// can customize the copy & paste mechanism.
/// To control copy/paste process the grid provides ICopyPaste interface and its default 
/// implementation in the CCopyPaste class.
///
/// \par Drag and Drop
///
/// The grid provides default Drag & drop implementation. It is possible to move selected lines inside one grid, 
/// between grids or between different applications. Moreover, at any moment the programmer can customize drag & drop 
/// behavior for his purposes. To indicate a dropping place the grid illuminates the lines between which data 
/// will be inserted. However in case of hierarchical data it is not enough because the level of hierarchy remains 
/// unknown. Our grid has a possibility to indicate this hierarchical level that allows you to clearly specify the 
/// user's wishes concerning inserting data into the desirable position.
/// All drag and drop steps are conducted though IDragDrop interface where they can be totally redefined.
///
///
/// \par Sorting
///
/// This is one of automatic services that allows you to greatly simplify programming and at the same 
/// time raising application quality. If data are updated (notification from Common::CDataObject::ISink 
/// interface or UpdateRow(), UpdateItem() or UpdateDataObject() functions call), the grid will automatically search 
/// a new position according to the sorting and hierarchy rules. Information about sorting is kept in the header. 
/// If the header is changed then sorting is also updated. 
/// Sorting can be single and multiple on one or several grid columns. Single click with the left mouse button 
/// on the header column switches the single sorting on. Second click inverts the direction of sorting. 
/// Multiple sorting is possible with mouse click while Ctrl key is hold. 
/// The second click with Ctrl key hold changes sorting direction in the column where the click was made. 
/// In order to cancel sorting it's enough to click on the header with the left mouse button and holding 
/// Shift key. Of course it is possible to control sorting by programming.
///
///
/// \par Data filtration
///
/// The grid allows to realize automatic data filtration. A grid line is considered filtered if the 
/// operator () of IGridFilter interface returns true. 
/// If the line is filtered then neither the line nor its children can be visible. The check for 
/// visibility is carried out at C++ object insertion, at its updating through 
/// Common::CDataObject::ISink interface, at UpdateRow(), UpdateItem() or UpdateDataObject() functions 
/// call and in some other cases.
/// 
///
/// \par Navigation and keyboard
///
/// All management over navigation is put to the external module that opens practically unlimited possibilities 
/// to manage received from the keyboard events. All navigation is collected in the implementation of 
/// IKeyManager interface. We offer the default implementation in the CKeyManager class. An object of this class 
/// can be got by calling CPreferences::GetDefaultKeyManager().
/// 
/// \par Printing
///
/// The grid allows you to print its content on a printer and also has a 'print preview' option. 
/// There are various levels for scaling and a possibility to choose either a portrait or a landscape mode.
///
/// \par Built in controls
///
/// Traditionally so called drawing of either control directly at the place of grid cells location is 
/// used to realize 'edit in place' possibilities. 
/// The controls themselves are created in the moment when 
/// the user clicks in a grid cell. Usually assortment of such controls is rather limited and depends 
/// on the grid's developer. Often the necessity to use real controls or controls of outside developers appears.
/// In such case it is required to keep a track of control location, its movements, size and visibility changes 
/// according to scrollbars location, filtration, cells size, position in a hierarchy and so on. Realization 
/// of the code positioning the control is a difficult and laborious task for the programmer. In order to 
/// simplify development of a resulting application the gird provides a convenient service for positioning 
/// practically any controls above the grid. The programmer just needs to assign the cell to which a control 
/// is to be attached and specify some control settings. Necessary to consider that the grid doesn't 
/// manage control's life time. The programmer takes all responsibility for creating and deleting 
/// controls. The grid only conducts control positioning relative to a grid cell.
///
///
/// \par Edit in place
///
/// The grid provides a very simple and useful positioning service for practically any MFC controls with navigation 
/// between them. In most cases, buttons, check boxes, and so on are not real controls but only their drawings. 
/// Basically, such 'controls' are supplied together with the grids and their number and functionality may be limited. 
/// Moreover such approach does not allow the use of already existing solutions or the third-party libraries. 
/// In case of real controls it is necessary to handle their location and size according to a number of elements in the grid, 
/// scrollbars' positions and also to manage their visibility (in case of filtration, expand/collapse and row 
/// deleting). This is not a trivial task and some difficulties may appear in the process of its realization. 
/// We offer the solution that permits you to attach pratically any MFC control to any cell and the grid will handle 
/// the positioning and visibility of the controls itself.
///
///
/// \par Serialization
///
/// Serialization in CArchive is a useful grid feature. A position, color of columns, colors, the user's preferences,
/// and so on are serialized. The grid is projected in such a way that new grid versions will be compatible with 
/// the archives made in previous ones. 
///
///
/// \par Thread safety
///
/// The grid was designed to work in a heavy multi-threaded environment. Practically all functions are thread-safe. 
/// They use both synchronous and as asynchronous mechanisms with or without blocking the calling thread. 
/// Inserting, removing, getting etc. work with blocking the calling thread. Any data updating are processed through 
/// the asynchronous mechanism. The calling thread does not wait for the end of updating operation. 
/// Much updating may occur at the same time. In this case, the dead-lock between %GUI and non-%GUI threads is 
/// not possible.
///
/// To display information in cells, the grid needs to retrieve values from your business objects. There are 
/// two thread-safe modes to do it:
/// - CPreferences::DirectCall: The grid asks for values in the %GUI thread each time when it needs to paint 
///   cells or scroll lines. It is up to you to protect your business data (the grid is thread safe). 
///   This mode is more productive than the other. It has low CPU loading and low memory consumption.
/// - CPreferences::MemoryBuffer: At each update the grid copies values and formats them in the calling thread 
/// and then no longer calls methods of your objects. The grid gets all information from the local cache. 
/// This mode is less productive but more secure.
/// 
/// \class CGrid Grid.h Dapfor/GUI/Grid.h
///
/// Read \ref DataObjectIntro "What is a data object?" article to get more information about binding.
///
/// The article \ref howtoinstall "How to install the MFC grid and compile the first application?" will explain all 
/// steps how to install and compile your first application.
///
//////////////////////////////////////////////////////////////////////////////////////////////
class DF_GUI_API CGrid : public CWnd
{
public:
    CGrid();
    virtual ~CGrid(void);

    // Addition, deleting, positioning, enumeration
    HITEM               Add(Common::CDataObject* pDO, HITEM hParent = GI_SCROLLABLE_ROOT, HITEM hInsertAfter = GI_LAST);
    bool                Delete(HITEM hItem);
    int                 Delete(const Common::CDataObject* pDO);
    bool                Delete(int nRow, ContextType contextType = ScrollableContext);
    void                DeleteAll(ContextType contextType = AllContexts);
    Handles             GetHandles(const Common::CDataObject* pDO, 
                                   bool                            visibleOnly, 
                                   ContextType                     contextType = AllContexts) const;
    


    bool                Attach(HITEM hItem, HITEM hParent, HITEM hInsertAfter = GI_LAST);
    bool                Detach(HITEM hItem);

    bool                Exists(HITEM hItem) const;
    bool                Exists(const Common::CDataObject* pDO) const;
    unsigned int        GetRowCount(ContextType contextType = ScrollableContext) const;
    void                ForEach(IForEach& functional, HITEM startFrom = GI_FIRST);
    void                ForEachVisible(IForEach& functional, HITEM startFrom = GI_FIRST);

    bool                MoveBefore(HITEM movingItem, HITEM parentPosition, HITEM position);
    bool                MoveAfter(HITEM movingItem, HITEM parentPosition, HITEM position);


    //Getting and setting formatted and non-formatted data
    Common::CValue GetValue(HITEM hItem, Common::FID fid) const;
    bool                SetValue(HITEM hItem, Common::FID fid, const Common::CValue& value);
    TString             GetFormattedStlString(HITEM hItem, Common::FID fid, const Common::CFormat* format = 0) const;
    bool                SetFormattedStlString(HITEM hItem, Common::FID fid, const TString& str, const Common::CFormat* format = 0);

#ifndef DF_NOT_MFC
    CString             GetFormattedMfcString(HITEM hItem, Common::FID fid, const Common::CFormat* format = 0) const;
    bool                SetFormattedMfcString(HITEM hItem, Common::FID fid, const CString& str, const Common::CFormat* format = 0);
#endif//DF_NOT_MFC


    //Functions to work with hierarchy
    bool                IsExpanded(HITEM hItem) const;
    Handles             GetChildren(HITEM hItem, bool visibleOnly) const;
    bool                HasChildren(HITEM hItem) const;
    int                 GetHierarchicalLevel(HITEM hItem) const;
    HITEM               GetParent(HITEM hItem) const;
    void                Expand(HITEM hItem, bool withChildren);
    void                Expand(const Common::CDataObject* pDO, bool withChildren);
    void                Collapse(HITEM hItem, bool withChildren);
    void                Collapse(const Common::CDataObject* pDO, bool withChildren);


    //Data objects and lines search in the grid
    int                 GetRow(HITEM hItem) const;
    int                 GetRow(const Common::CDataObject* pDO) const;
    ContextType         GetContextType(HITEM hItem) const;
    ContextType         GetContextType(const Common::CDataObject* pDO) const;
    HITEM               GetItem(int nRow, ContextType contextType = ScrollableContext) const;
    HITEM               GetItem(const Common::CDataObject* pDO) const;
    HITEM               GetItem(CPoint point) const;
    HITEM               GetPreviousItem(HITEM hItem, bool jumpBetweenContexts = true) const;
    HITEM               GetNextItem(HITEM hItem, bool jumpBetweenContexts = true) const;
    Common::CDataObject*       GetDataObject(HITEM hItem);
    const Common::CDataObject* GetDataObject(HITEM hItem) const;
    Common::CDataObject*       GetDataObject(int nRow, ContextType contextType = ScrollableContext);
    const Common::CDataObject* GetDataObject(int nRow, ContextType contextType = ScrollableContext) const;

    

    //Visibility, placement and bounds
    int                 GetFirstVisibleRow() const;
    int                 GetLastVisibleRow(ContextType contextType = ScrollableContext) const;
    
    void                SetFirstVisibleRow(int nRow);
    void                SetFirstVisibleItem(HITEM hItem);
    void                SetFirstVisibleItem(const Common::CDataObject* pDO);
    
    void                EnsureVisible(int nRow);
    void                EnsureVisible(HITEM hItem);
    void                EnsureVisible(const Common::CDataObject* pDO);
    
    bool                IsVisible(HITEM hItem) const;
    bool                IsVisible(const Common::CDataObject* pDO) const;
    
    bool                IsInVisibleRect(HITEM hItem);
    bool                IsInVisibleRect(const Common::CDataObject* pDO);
    bool                IsInVisibleRect(int nRow, ContextType contextType);

    CRect               GetContextVisibleRect(ContextType contextType) const;
    CRect               GetContextVirtualRect(ContextType contextType) const;
    
    CRect               GetLineVisibleRect(HITEM hItem) const;
    CRect               GetLineVisibleRect(const Common::CDataObject* pDO) const;
    
    CRect               GetLineVirtualRect(HITEM hItem) const;
    CRect               GetLineVirtualRect(const Common::CDataObject* pDO) const;
    
    CRect               GetCellVisibleRect(HITEM hItem, Common::FID fid) const;
    CRect               GetCellVisibleRect(const Common::CDataObject* pDO, Common::FID fid) const;
    
    CRect               GetCellVirtualRect(HITEM hItem, Common::FID fid) const;
    CRect               GetCellVirtualRect(const Common::CDataObject* pDO, Common::FID fid) const;

    int                 GetLinesPerPage(bool forward = true) const;
    int                 GetItemHeight(HITEM hItem) const;


    //Selection
    void                AllowSelection(bool bEnable);
    bool                CanSelect() const;
    void                AllowMultiSelection(bool bEnable);
    bool                CanMultiSelect() const;
    void                AllowSelectContext(ContextType contextType = AllContexts);
    bool                CanSelectContext(ContextType contextType) const;
    int                 GetSelectedCount() const;
    bool                Select(int nRow, ContextType contextType, bool bSelect);
    bool                Select(HITEM hItem, bool bSelect);
    bool                Select(const Common::CDataObject* pDO, bool bSelect);
    Selection           GetSelection(ContextType contextType) const;
    bool                IsSelected(int nRow, ContextType contextType) const;
    bool                IsSelected(HITEM hItem) const;
    bool                IsSelected(const Common::CDataObject* pDO) const;
    void                SelectAll();
    void                ClearSelection();
    void                AllowMouseTrackSelection(bool bEnable);
    bool                CanMouseTrackSelection() const;
        
    //Focus
    void                AllowFocusContext(ContextType contextType = AllContexts);
    bool                CanFocusContext(ContextType contextType) const;
    bool                SetFocusedRow(int nRow, ContextType contextType);
    int                 GetFocusedRow() const;
    ContextType         GetFocusedContext() const;
    


    //Updating
    void                UpdateRow(int nRow, ContextType contextType, Common::FID fid);
    void                UpdateItem(HITEM hItem, Common::FID fid);
    void                UpdateDataObject(const Common::CDataObject* pDO, Common::FID fid);
    int                 GetTimeElapsedSinceLastUpdate(int nRow, ContextType contextType, Common::FID fid) const;
    int                 GetTimeElapsedSinceLastUpdate(HITEM hItem, Common::FID fid) const;
    
    void                InvalidateRow(int nRow, ContextType contextType, int spanTime = 0);
    void                InvalidateRow(HITEM hItem, int spanTime = 0);
    void                InvalidateRow(const Common::CDataObject* pDO, int spanTime = 0);

    void                InvalidateCell(int nRow, ContextType contextType, Common::FID fid, int spanTime = 0);
    void                InvalidateCell(HITEM hItem, Common::FID fid, int spanTime = 0);
    void                InvalidateCell(const Common::CDataObject* pDO, Common::FID fid, int spanTime = 0);


    //Notifications
    void                AllowNotifications(bool bAllow);
    bool                IsNotificationsAllowed() const;

    //////////////////////////////////////////////////////////
    //Appearance
    //////////////////////////////////////////////////////////
    CAppearance&        GetAppearance();
    const CAppearance&  GetAppearance() const;
    void                SetAppearance(const CAppearance& appearance);

    //Background images
    const CBackgroundImage& GetBackgroundImage() const;
    void                    SetBackgroundImage(const CBackgroundImage& settings);


    //Fonts
    void                SetFont(CFont* font, BOOL bRedraw = TRUE);
    CFont*              GetFont() const;


    //ImageList to draw images in cell
    CImageList*         SetImageList(CImageList* imageList, bool stretchImage = false);
    CImageList*         GetImageList() const;
    void                AllowStretchImageInCell(bool stretch);
    bool                CanStretchImageInCell() const;


    //Context separator
    void                AllowDrawContextSeparator(bool bDraw);
    bool                CanDrawContextSeparator() const;
    void                SetContextSeparatorHeight(int nHeight);
    int                 GetContextSeparatorHeight() const;

    //Highlighting
    void                      AllowHighlight(bool bAllow);
    bool                      CanHighlight() const;
    const CHighlightSettings& GetHighlightSettings() const;
    CHighlightSettings&       GetHighlightSettings();
    void                      SetHighlightSettings(const CHighlightSettings& settings);


    //Custom draw & painting
    ICustomDraw*        SetCustomDraw(ICustomDraw* customDraw);
    ICustomDraw*        GetCustomDraw() const;
    void                SetPaintFilter(UINT paintFilter = defaultPaintFilter);
    UINT                GetPaintFilter() const;
    void                SetFlickerFreeDrawing(bool enable);
    bool                GetFlickerFreeDrawing() const;
    int                 GetOptimalRowHeight(ContextType contextType) const;
    int                 GetOptimalCellWidth(int nRow, ContextType contextType, Common::FID fid) const;
    int                 GetOptimalColumnWidth( ContextType contextType, Common::FID fid) const;
    

    // Headers
    CHeader*            SetHeader(CHeader* newHeader);
    CHeader*            GetHeader();
    const CHeader*      GetHeader() const;
    void                ShowHeader();
    void                HideHeader();
    bool                IsHeaderVisible() const;
    virtual void        ShowHeaderDialog();
    const CColumn*      GetColumn(CPoint pt) const;
    const CColumn*      GetNavigationColumn() const ;
    const CColumn*      GetHierarchicalColumn() const ;
    void                AdjustColumnByFid(Common::FID fid);
    void                AdjustColumnByIndex(int index);
    void                AdjustAllColumns();
    int                 GetOptimalHeaderHeight() const;
    void                EnsureColumnVisible(Common::FID fid);


    // Tooltips
    void                    EnableTooltip(bool enable = true);
    bool                    IsTooltipEnabled() const;
    const ITooltipCtrl*     GetTooltipCtrl();
    ITooltipCtrl*           SetTooltipCtrl(ITooltipCtrl* control);
    const ITooltip*         GetTooltip() const;
    ITooltip*               SetTooltip(ITooltip* tooltip);
    const CTooltipSettings& GetTooltipSettings() const;
    CTooltipSettings&       GetTooltipSettings();
    void                    SetTooltipSettings(const CTooltipSettings& settings);

    //Displays partially visible text of small cells
    void                    EnableInPlaceTooltip(bool bEnable = true);
    bool                    IsInPlaceTooltipEnabled() const;
    CTooltipSettings&       GetInPlaceTooltipSettings();
    void                    SetInPlaceTooltipSettings(const CTooltipSettings& settings);
    const ITooltipCtrl*     GetInPlaceTooltipCtrl() const;
    ITooltipCtrl*           SetInPlaceTooltipCtrl(ITooltipCtrl* control);


    //Copy/cut/paste
    bool                      Copy();
    bool                      Cut();
    bool                      Paste();
    bool                      Paste(HITEM hParent, HITEM hInsertAfter);
    ICopyPaste*               SetCopyPaste(ICopyPaste* copyPaste);
    ICopyPaste*               GetCopyPaste() const;
    
    const CCopyPasteSettings& GetCopyPasteSettings() const;
    CCopyPasteSettings&       GetCopyPasteSettings();
    void                      SetCopyPasteSettings(const CCopyPasteSettings& settings);
    
    void                      AllowCopyPaste(bool bAllow);
    bool                      IsCopyPasteAllowed() const;


    //Drag & drop
    IDragDrop*               SetDragDrop(IDragDrop* dragDrop);
    IDragDrop*               GetDragDrop() const;
    const CDragDropSettings& GetDragDropSettings() const;
    CDragDropSettings&       GetDragDropSettings();
    void                     SetDragDropSettings(const CDragDropSettings& settings);
    void                     AllowDragAndDrop(bool bAllow);
    bool                     IsDragAndDropAllowed() const;
    CDragImage               CreateDragImage(const Handles& handles, ContextType contextType, const CHeader* header) const;

    
    //Sorting
    void                AllowSort(bool bEnable);
    bool                CanSort() const;
    void                AllowMultiSort(bool bEnable);
    bool                CanMultiSort() const;
    void                SetSortedContext(ContextType contextType = AllContexts);
    ContextType         GetSortedContext() const;
    void                ResetSort( bool redraw = true );
    void                SetSort(int level, CSortInfo sortInfo, bool applyImmediatelly = true);
    CSortInfo           GetSort(int level) const;
    void                UpdateSort();
    int                 GetSortLevels() const;

    
    //Data filtration
    IGridFilter*        SetFilter(IGridFilter* filter);
    IGridFilter*        GetFilter() const;
    void                UpdateFilter();


    
    //Keyboard management
    IKeyManager*        SetKeyManager(IKeyManager* keyManager);
    IKeyManager*        GetKeyManager() const;


    //Navigation
    void                AllowNavigation(ContextType contextType = AllContexts);
    bool                CanNavigate(ContextType contextType) const;
    void                NavigateUp(bool bShift, bool bCtrl);
    void                NavigateDown(bool bShift, bool bCtrl);
    void                NavigatePageUp(bool bShift, bool bCtrl);
    void                NavigatePageDown(bool bShift, bool bCtrl);
    void                NavigateHome(bool bShift, bool bCtrl);
    void                NavigateEnd(bool bShift, bool bCtrl);

    void                ResetScroll();
    

    // Printing operations
    void                PrintPreview(const CPrintSettings& settings);
    void                Print(const CPrintSettings& settings, CPrintInfo* printInfo = 0);




    //Embedded controls
    CEmbeddedControlAdapter* AttachControl(CWnd* control, HITEM hItem, Common::FID fid, const CEmbeddedControlParams& params = CEmbeddedControlParams());
    bool                     DetachControl(CWnd* control);
    bool                     DetachControl(HITEM hItem, Common::FID fid, bool bDestroy);
    bool                     HasControl(HITEM hItem, Common::FID fid) const;
    bool                     HasControl(int nRow, ContextType contextType, Common::FID fid) const;

    
    //Edit in place
    IEditInPlace*               SetEditInPlace(IEditInPlace* editInPlace);
    IEditInPlace*               GetEditInPlace() const;
    bool                        CanStartEditInPlace(HITEM hItem, Common::FID fid, IEditInPlace::StartReason reason) const;
    bool                        StartEditInPlace(HITEM hItem, Common::FID fid, IEditInPlace::StartReason reason = IEditInPlace::ActivateOnUserStart);
    bool                        StopEditInPlace(IEditInPlace::StopReason reason, bool postMessage = false);
    CWnd*                       GetEditInPlaceControl() const;
    const CEditInPlaceSettings& GetEditInPlaceSettings() const;
    CEditInPlaceSettings&       GetEditInPlaceSettings();
    void                        SetEditInPlaceSettings(const CEditInPlaceSettings& settings);

    //Serialization
    virtual void Serialize(CArchive& ar);
    static int   GetSerializeVertion();

protected:
    //{{AFX_MSG(CGrid)
    afx_msg int     OnCreate(LPCREATESTRUCT lpCreateStruct);
    afx_msg void    OnDestroy();
    afx_msg void    OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
    afx_msg void    OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
    afx_msg LPARAM  OnDispatchTSMessage(WPARAM wParam, LPARAM lParam);
    afx_msg void    OnPaint();
    afx_msg BOOL    OnEraseBkgnd(CDC* pDC);
    afx_msg void    OnSize(UINT nType, int cx, int cy);
    afx_msg void    OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
    afx_msg void    OnLButtonDown(UINT nFlags, CPoint point);
    afx_msg BOOL    OnMouseWheel(UINT nFlags, short zDelta, CPoint pt);
    afx_msg void    OnTimer(UINT id);
    afx_msg LRESULT OnSetFont(WPARAM wParam, LPARAM lParam);
    afx_msg LRESULT OnGetFont(WPARAM wParam, LPARAM lParam);
    //}}AFX_MSG

    //{{AFX_VIRTUAL(CGrid)
    virtual LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam);
    virtual BOOL    OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo);
    virtual BOOL    PreCreateWindow(CREATESTRUCT& cs);
    virtual void    PreSubclassWindow();
    virtual BOOL    PreTranslateMessage(MSG* pMsg);
    //}}AFX_VIRTUAL

private:
    DECLARE_MESSAGE_MAP()
    DECLARE_SERIAL(CGrid)

    class CGridImpl; 

private:
    CGrid(const CGrid&);
    CGrid& operator = (const CGrid&);
    CGridImpl* m_Impl;
};

}//namespace GUI

}//namespace Dapfor

