/////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright 2007-2008 Dapfor
//                                                                         
// This file is part of the Dapfor software, and may only be used, modified, and distributed 
// under the terms of the Dapfor Software License, Version 1.0.
// (See accompanying file lisense.txt or open the license at www.dapfor.com/license_agreement.aspx)
// By continuing to use, modify, or distribute this file you indicate that you have read the license 
// and understand and accept it fully.
/////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include <Dapfor/GUI/GUI_Export.h>
#include <Dapfor/GUI/Types.h>
#include <Dapfor/GUI/IAutoDelete.h>

#include <Dapfor/GUI/HeaderPaintInfo.h>
#include <Dapfor/GUI/ColumnPaintInfo.h>
#include <Dapfor/GUI/PaintContext.h>
#include <Dapfor/GUI/GridLine.h>
#include <Dapfor/GUI/GridCell.h>
#include <Dapfor/GUI/Column.h>


namespace Dapfor
{
namespace GUI
{

class ILinePaintFormat;
class ICellPaintFormat;
class IHeaderPaintFormat;



//////////////////////////////////////////////////////////////////////////
///\brief Performs custom drawing in CGrid.
///
/// It is designed for customization of drawing of all elements in the grid. 
/// This interface is the central place through which all information is passed at each grid element drawing. 
/// The grid functions are divided into three groups. The first group is for drawing a header, 
/// the second one is for drawing lines and cells and the third returns optimal lines, columns 
/// and cells sizes calculated based upon the grid settings, fonts' size, size of the text displayed 
/// in the cells, hierarchy and so on.
///
/// The interface itself is not designed for direct manipulating with "device context" although 
/// nothing prevents to do so. Originally specialized formats were supposed to do this. For painting a header 
/// IHeaderPaintFormat and its default realization CHeaderPaintFormat are used. This format only calculates 
/// columns size and allocation in the grid and hands over their painting through ICustomDraw to 
/// IColumnPaintFormat. Neither IHeaderPaintFormat nor ICustomDraw conducts direct columns painting.
///
/// The same mechanism is used for painting lines and cells. ICustomDraw passes all necessary information 
/// to ILinePaintFormat that in its turn calculates cells location in the grid and through ICustomDraw 
/// passes the calculated information to ICellPaintFormat which directly performs painting. 
///
/// Such approach allows to collect all graphical information at one place and to totally redefine 
/// grid painting at any moment. When it is necessary to redefine colors, font, icon, to forbid displaying 
/// of selection in some separate cells then ICustomDraw is the best place to do so.
/// As for calculation of the best height or width for a header, lines and cells the formats can do this best 
/// of all because they know what way information will be presented on the screen.
///
/// \version <b>1.0.0</b> Initial version
///
/// \class  ICustomDraw ICustomDraw.h Dapfor/GUI/ICustomDraw.h
//////////////////////////////////////////////////////////////////////////
class DF_GUI_API ICustomDraw : virtual public IAutoDelete
{
public:
    ///Called to draw a header in CGrid.
    virtual void DrawHeader(CHeaderPaintInfo& paintInfo, const CPaintContext& paintContext, UINT paintFilter) = 0;

    ///Called to draw a column in CGrid.
    virtual void DrawColumn(CColumnPaintInfo& paintInfo, const CPaintContext& paintContext, UINT paintFilter) = 0;

    /// Called to draw a line. 
    virtual void DrawLine(CGridLine& line, const CPaintContext& paintContext, UINT paintFilter) = 0;

    /// Called to draw a cell. 
    virtual void DrawCell(CGridCell& cell, const CPaintContext& paintContext, UINT paintFilter) = 0;

    /// Provides a default line paint format. The user can specify his preferred format for each line in CGrid.
    virtual const ILinePaintFormat& GetLinePaintFormat(const Common::CDataObject* pDO, HITEM hItem, ContextType contextType, int nRow) const = 0;

    /// Provides a default cell paint format. The user can specify his preferred format for each cell in CGrid.
    virtual const ICellPaintFormat& GetCellPaintFormat(const CColumn* column, const Common::CDataObject* pDO, HITEM hItem, ContextType contextType, int nRow) const = 0;

    /// Calculates an optimal height of a header.
    virtual int GetOptimalHeaderHeight(CHeaderPaintInfo& paintInfo, const CPaintContext& paintContext, UINT paintFilter) const = 0;

    /// Calculates an optimal height of a row.
    virtual int GetOptimalRowHeight(CFont* font, CDC& dc) const = 0;
    
    /// Calculates an optimal width of a cell taking into account a font, hierarchy, text size...
    virtual int GetOptimalCellWidth(CGridCell& cell, const CPaintContext& paintContext, UINT paintFilter) const = 0;
};

}//namespace GUI

}//namespace Dapfor



